const { parsers: babelParsers } = require("prettier/parser-babel")

/**
 * 词典序比较函数
 * @param {string} a - 第一个字符串
 * @param {string} b - 第二个字符串
 * @returns {number} - 比较结果
 */
function lexicalSort(a, b) {
  return a > b ? 1 : a < b ? -1 : 0
}

/**
 * 移除对象中的重复键，可选择是否按词典序排序
 * @param {*} obj - 输入对象
 * @param {Object} options - 配置选项
 * @param {boolean} options.enableSortJSON - 是否启用键排序
 * @returns {*} - 处理后的对象
 */
function removeDuplicateKeys(obj, options = { enableSortJSON: true }) {
  // 处理基本类型和 null
  if (typeof obj !== "object" || obj === null) {
    return obj
  }

  // 处理数组
  if (Array.isArray(obj)) {
    return obj.map(item => removeDuplicateKeys(item, options))
  }

  // 处理对象
  const result = {}
  const keys = Object.keys(obj)
  
  // 从后向前遍历，保留最后出现的键值对
  for (let i = keys.length - 1; i >= 0; i--) {
    const key = keys[i]
    if (!(key in result)) {
      result[key] = removeDuplicateKeys(obj[key], options)
    }
  }

  // 根据选项决定是否排序
  const resultKeys = options.enableSortJSON ? Object.keys(result).sort(lexicalSort) : Object.keys(result)
  return resultKeys.reduce((acc, key) => ({ ...acc, [key]: result[key] }), {})
}

/**
 * 解析 JSON 文本
 * @param {string} text - JSON 文本
 * @param {Object} options - 配置选项
 * @returns {Object} - 解析后的对象
 * @throws {Error} - 解析错误
 */
function parseJSON(text, options = {}) {
  try {
    const parsed = JSON.parse(text)
    return removeDuplicateKeys(parsed, { 
      enableSortJSON: options.enableSortJSON !== false
    })
  } catch (error) {
    throw new Error(`Failed to parse JSON: ${error.message}`)
  }
}

module.exports = {
  languages: [
    {
      name: "JSON No Duplicates",
      parsers: ["json-no-duplicates"],
      extensions: [".json"],
      filenames: ["package.json", "composer.json"],
      linguistLanguageId: 174,
      vscodeLanguageIds: ["json", "jsonc"]
    }
  ],
  parsers: {
    "json-no-duplicates": {
      parse: parseJSON,
      astFormat: "json-no-duplicates-ast",
      locStart: () => 0,
      locEnd: () => 0
    }
  },
  printers: {
    "json-no-duplicates-ast": {
      print: (path, options) => {
        const node = path.getValue()
        return JSON.stringify(node, null, options.tabWidth)
      },
      embed: false
    }
  },
  options: {
    enableSortJSON: {
      type: "boolean",
      category: "Global",
      default: true,
      description: "Sort JSON object keys lexically"
    }
  },
  defaultOptions: {
    tabWidth: 2,
    enableSortJSON: true
  }
}
